<?php

namespace DsbNext\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class DashboardTAController{
    
    protected $crudService;
    protected $service;
    
    public function __construct($connection, $environment, $util, $crudService, $service) {
        $this->crudService     = $crudService;
        $this->connection      = $connection;
        $this->environment     = $environment;
        $this->util            = $util;
        $this->service         = $service;
    }
    
    public function previousDayAFDeliveryTA(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        
        $data = $this->crudService->getFilteredDataSet('previousDayAFDeliveryTA', $conditions)->getRows();
        $response->addDataSet(new DataSet("previousDayAFDeliveryTA", $data));
    }
    
    public function afApprovalTracking(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        
        $data = $this->crudService->getFilteredDataSet('afApprovalTracking', $conditions)->getRows();
        $response->addDataSet(new DataSet("afApprovalTracking", $data));
    }
    
    public function afApprovalTrackingGrid(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $variables = array();
        
        array_push($variables ,
            array (
                "columnName" => "INICIOMES",
                "operator" => "=",
                "value" => $filter['INICIOMES']
            ),
            array (
                "columnName" => "FIMMES",
                "operator" => "=",
                "value" => $filter['FIMMES']
            ),
        );
        $route = $filter['route'];
        switch($route){
            case "CONFIRMADA":
                $data = $this->crudService->getFilteredDataSet('getAFsWithConfirmation', array_merge($conditions, $variables))->getRows();
                break;
            case "NAOCONFIRMADA":
                $data = $this->crudService->getFilteredDataSet('getAFsWithoutConfirmation', array_merge($conditions, $variables))->getRows();
                break;
            case "COMNF":
                $data = $this->crudService->getFilteredDataSet('getAFsWithNF', array_merge($conditions, $variables))->getRows();
                break;
            case "SEMNF":
                $data = $this->crudService->getFilteredDataSet('getAFsWithoutNF', array_merge($conditions, $variables))->getRows();
                break;
        }
        $response->addDataSet(new DataSet("AfApprovalTrackingGrid", $data));
    }
    
    public function aFDeliveryLastWeekTA(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        
        $data = $this->crudService->getFilteredDataSet('aFDeliveryLastWeekTA', $conditions)->getRows();
        $response->addDataSet(new DataSet("aFDeliveryLastWeekTA", $data));
    }
    
    public function aFDeliveryLastWeekGrid(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $variables = array();
        
        array_push($variables ,
            array (
                "columnName" => "SELECTED",
                "operator" => "=",
                "value" => $filter['SELECTED']
            )
        );
        
        $data = $this->crudService->getFilteredDataSet('aFDeliveryLastWeekGrid', array_merge($conditions, $variables))->getRows();
        $response->addDataSet(new DataSet("aFDeliveryLastWeekGrid", $data));
    }
    
    public function aFDeliveryLastYear(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        
        $data = $this->crudService->getFilteredDataSet('aFDeliveryLastYear', $conditions)->getRows();
        $response->addDataSet(new DataSet("aFDeliveryLastYear", $data));
    }
    
    public function aFGeneral(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        $data = $this->crudService->getFilteredDataSet('aFGeneral', $conditions)->getRows();
        $response->addDataSet(new DataSet("aFGeneral", $data));  
    }
    
    public function aFBuyer(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        $data = $this->crudService->getFilteredDataSet('aFBuyer', $conditions)->getRows();
        $response->addDataSet(new DataSet("aFBuyer", $data));  
    }
    
    public function aFConfSupplier(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        $data = $this->crudService->getFilteredDataSet('aFConfSupplier', $conditions)->getRows();
        $response->addDataSet(new DataSet("aFConfSupplier", $data));  
    }
    
    public function drillDownAFBuyerFirstLevel($conditions, $operador, $response) {
        $params = [
                'CDFILCENTCMP' => $conditions[0]['value'],
                'CDOPERADOR' => $operador
        ];
        try {
            $data = $this->service->drillDownAFBuyerFirstLevel($params);
            $response->addDataSet(new DataSet("drillDownPieChart", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function drillDownAFGeneralFirstLevel($conditions, $idRecebimento, $response) {
        $params = [
                'CDFILCENTCMP' => $conditions[0]['value'],
                'IDCONFRECEBIMENTO' => $idRecebimento
        ];
        try {
            $data = $this->service->drillDownAFGeneralFirstLevel($params);
            $response->addDataSet(new DataSet("drillDownPieChart", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    private function drillDownAFGeneralSecondLevel($conditions, $idRecebimento, $operador, $response){
        $params = array(
            "CDOPERADOR" => $operador,
            "CDFILCENTCMP" => $conditions[0]['value'],
            "IDCONFRECEBIMENTO" => $idRecebimento
        );
        try {
            $data = $this->service->drillDownAFGeneralSecondLevel($params);
            $response->addDataSet(new DataSet("drillDownPieChart", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function drillDownAFGeneralThirdLevel($conditions, $idRecebimento, $operador, $regiao, $response) {
        $params = array(
            "NMREGIAOGEO" => $regiao,
            "CDOPERADOR" => $operador,
            "CDFILCENTCMP" => $conditions[0]['value'],
            "IDCONFRECEBIMENTO" => $idRecebimento
        );
        try {
            $data = $this->service->drillDownAFGeneralThirdLevel($params);
            $response->addDataSet(new DataSet("drillDownPieChart", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function drillDownAFDeliveryLastYearFirstLevel($conditions, $ENDDATE, $STARTDATE, $response) {
        $params = array(
            "ENDDATE" => $ENDDATE,
            "STARTDATE" => $STARTDATE,
            "CDFILCENTCMP" => $conditions[0]['value']
        );
        try {
            $data = $this->service->drillDownAFDeliveryLastYearFirstLevel($params);
            $response->addDataSet(new DataSet("drillDownStackedChart", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    private function drillDownAFDeliveryLastYearSecondLevel($conditions, $ENDDATE, $STARTDATE, $operador, $response){
        $params = array(
            "CDOPERADOR" => $operador,
            "CDFILCENTCMP" => $conditions[0]['value'],
            "ENDDATE" => $ENDDATE,
            "STARTDATE" => $STARTDATE
        );
        try {
            $data = $this->service->drillDownAFDeliveryLastYearSecondLevel($params);
            $response->addDataSet(new DataSet("drillDownStackedChart", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function drillDownAFDeliveryLastYearThirdLevel($conditions, $ENDDATE, $STARTDATE, $operador, $regiao, $response) {
        $params = array(
            "NMREGIAOGEO" => $regiao,
            "CDOPERADOR" => $operador,
            "CDFILCENTCMP" => $conditions[0]['value'],
            "ENDDATE" => $ENDDATE,
            "STARTDATE" => $STARTDATE
        );
        try {
            $data = $this->service->drillDownAFDeliveryLastYearThirdLevel($params);
            $response->addDataSet(new DataSet("drillDownStackedChart", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function drillDownPieChart(Request\Filter $request, Response $response) {
        $data = $request->getParameters();
        $IDCONFRECEBIMENTO = isset($data['IDCONFRECEBIMENTO'])? $data['IDCONFRECEBIMENTO'] : null;
        $CDOPERADOR = isset($data['operador'])? $data['operador']: null;
        $REGIAO = isset($data['regiao'])? $data['regiao'] : null;

        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $level = $data['level'];
        switch($level){
            case 0:
                if ($IDCONFRECEBIMENTO != null) {
                    $this->drillDownAFGeneralFirstLevel($conditions, $IDCONFRECEBIMENTO, $response); 
                } else {
                    $this->drillDownAFBuyerFirstLevel($conditions, $CDOPERADOR, $response);    
                }
                break;
            case 1:
                $this->drillDownAFGeneralSecondLevel($conditions, $IDCONFRECEBIMENTO, $CDOPERADOR, $response);
                break;
            case 2:
                $this->drillDownAFGeneralThirdLevel($conditions, $IDCONFRECEBIMENTO, $CDOPERADOR, $REGIAO, $response);
        }
    }
    
    public function drillDownStackedChart(Request\Filter $request, Response $response) {
        $data = $request->getParameters();
        $ENDDATE = isset($data['ENDDATE'])? $data['ENDDATE'] : null;
        $STARTDATE = isset($data['STARTDATE'])? $data['STARTDATE']: null;
        $CDOPERADOR = isset($data['CDOPERADOR'])? $data['CDOPERADOR']: null;
        $REGIAO = isset($data['NMREGIAOGEO'])? $data['NMREGIAOGEO'] : null;

        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $level = $data['level'];
        switch($level){
            case 0:
                $this->drillDownAFDeliveryLastYearFirstLevel($conditions, $ENDDATE, $STARTDATE, $response);
                break;
            case 1:
                $this->drillDownAFDeliveryLastYearSecondLevel($conditions, $ENDDATE, $STARTDATE, $CDOPERADOR, $response);
                break;
            case 2:
                $this->drillDownAFDeliveryLastYearThirdLevel($conditions, $ENDDATE, $STARTDATE, $CDOPERADOR, $REGIAO, $response);
        }
    }
    
    /**
     * This function retrieves and filters data for the AFGeneralTAGrid.
     *
     * It first prepares conditions based on the parameters array ($params) which currently contains
     * 'CDFILCENTCMP'. The conditions are prepared using the utility method `prepareConditions`.
     * 
     * The function then populates an array ($variables) with filtering criteria retrieved from the
     * front-end. These filters are applied to columns: 'OPERADOR', 'IDCONFRECEBIMENTO', 'NMESTADO', 
     * and 'NMREGIAOGEO'.
     *
     * The merged array of conditions and variables is passed to the `getFilteredDataSet` method
     * of the `crudService`, which retrieves the filtered data from the 'AFGeneralTAGrid' table.
     * 
     * The resulting data is added to the response as a dataset named 'AFGeneralTAGridGroup'.
     *
     * @param Request\Filter $request The object that receives data from the caller.
     * @param Response $response The object that sends data back to the caller.
     *
     * @return void
     *
     * @throws Exception If an error occurs during data retrieval.
     * 
     * @access public
     * @author Arthur Gustavo
     */
    
    public function aFGeneralTAGrid(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $variables = array();
        
        array_push($variables ,
            array (
                "columnName" => "OPERADOR",
                "operator" => "=",
                "value" => $filter['OPERADOR']
            ),
            array (
                "columnName" => "IDCONFRECEBIMENTO",
                "operator" => "=",
                "value" => $filter['IDCONFRECEBIMENTO']
            ),
            array (
                "columnName" => "NMESTADO",
                "operator" => "=",
                "value" => $filter['NMESTADO']
            ),
            array (
                "columnName" => "NMREGIAOGEO",
                "operator" => "=",
                "value" => $filter['NMREGIAOGEO']
            )
        );
        
        $data = $this->crudService->getFilteredDataSet('aFGeneralTAGrid', array_merge($conditions, $variables))->getRows();
        $response->addDataSet(new DataSet("aFGeneralTAGridGroup", $data));
    }
    
    public function aFBuyerGrid(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $variables = array();
        
        array_push($variables ,
            array (
                "columnName" => "OPERADOR",
                "operator" => "=",
                "value" => $filter['OPERADOR']
            ),
            array (
                "columnName" => "IDCONFRECEBIMENTO",
                "operator" => "=",
                "value" => $filter['IDCONFRECEBIMENTO']
            ),
            array (
                "columnName" => "NMESTADO",
                "operator" => "=",
                "value" => $filter['NMESTADO']
            ),
            array (
                "columnName" => "NMREGIAOGEO",
                "operator" => "=",
                "value" => $filter['NMREGIAOGEO']
            )
        );
        
        $data = $this->crudService->getFilteredDataSet('aFBuyerGrid', array_merge($conditions, $variables))->getRows();
        $response->addDataSet(new DataSet("aFBuyerGridGroup", $data));
    }
    
    public function aFConfSupplierGrid(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $variables = array();
        
        array_push($variables ,
            array (
                "columnName" => "OPERADOR",
                "operator" => "=",
                "value" => $filter['OPERADOR']
            ),
            array (
                "columnName" => "IDCONFRECEBIMENTO",
                "operator" => "=",
                "value" => $filter['IDCONFRECEBIMENTO']
            ),
            array (
                "columnName" => "NMESTADO",
                "operator" => "=",
                "value" => $filter['NMESTADO']
            ),
            array (
                "columnName" => "NMREGIAOGEO",
                "operator" => "=",
                "value" => $filter['NMREGIAOGEO']
            )
        );
        
        $data = $this->crudService->getFilteredDataSet('aFConfSupplierGrid', array_merge($conditions, $variables))->getRows();
        $response->addDataSet(new DataSet("aFConfSupplierGridGroup", $data));
    }
    
    public function aFDeliveryLastYearGrid(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $variables = array();
        
        array_push($variables ,
            array (
                "columnName" => "ENDDATE",
                "operator" => "=",
                "value" => $filter['ENDDATE']
            ),
            array (
                "columnName" => "STARTDATE",
                "operator" => "=",
                "value" => $filter['STARTDATE']
            ),
            array (
                "columnName" => "CDOPERADOR",
                "operator" => "=",
                "value" => $filter['CDOPERADOR']
            ),
            array (
                "columnName" => "NMREGIAOGEO",
                "operator" => "=",
                "value" => $filter['NMREGIAOGEO']
            ),
            array (
                "columnName" => "NMESTADO",
                "operator" => "=",
                "value" => $filter['NMESTADO']
            )
        );
        
        $data = $this->crudService->getFilteredDataSet('aFDeliveryLastYearGrid', array_merge($conditions, $variables))->getRows();
        $response->addDataSet(new DataSet("aFDeliveryLastYearGridGroup", $data));
    }
    
    public function getGeoChartData(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $ENDDATE = isset($filter['ENDDATE'])? $filter['ENDDATE'] : null;
        $STARTDATE = isset($filter['STARTDATE'])? $filter['STARTDATE']: null;
        
        $variables = array(
            "CDFILCENTCMP" => $conditions[0]['value'],
            "ENDDATE" => $ENDDATE,
            "STARTDATE" => $STARTDATE
        );
        
        try {
            $data = $this->service->getGeoChartData($variables);
            $response->addDataSet(new DataSet("GeoChartData", $data));
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function averageTimeAFGrid(Request\Filter $request, Response $response) {
        $params = ['CDFILCENTCMP'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getParameters();
        $variables = array();
        
        array_push($variables ,
            array (
                "columnName" => "SGESTADO",
                "operator" => "=",
                "value" => $filter['SGESTADO']
            ),
            array (
                "columnName" => "DTINI",
                "operator" => "=",
                "value" => $filter['DTINI']
            ),
            array (
                "columnName" => "DTFIN",
                "operator" => "=",
                "value" => $filter['DTFIN']
            )
        );
        
        $data = $this->crudService->getFilteredDataSet('averageTimeAFGrid', array_merge($conditions, $variables))->getRows();
        $response->addDataSet(new DataSet("averageTimeAFGridGroup", $data));
    }
}