import { Metadata, Http } from '@zeedhi/core';
import { Modal, ModalService, SelectTree } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class AverageTimeAFModalController {
  private AverageTimeAFModal!: Modal;

  public async createAverageTimeAFModal() {
    this.AverageTimeAFModal = await ModalService.createFromJson('/modals/averageTimeAFGridModal', true);
  }

  public destroyAverageTimeAFModal() {
    this.AverageTimeAFModal?.destroy();
  }
  
  public async openAverageTimeAFModal(area) {
    await this.AverageTimeAFModal.show();
    const gridGroup = Metadata.getInstance<TekGrid>('averageTimeAFGridGroup');
    const selectedValue = Metadata.getInstance<SelectTree>('selectTreeDatasource').selectValue;
    const defaultDate = Metadata.getInstance<SelectTree>('selectTreeDatasource').datasource.data[selectedValue].label;
    const [start, end] = defaultDate.split(' a ');
    gridGroup.title = `Estado Selecionado: ${this.getNomeEstado(area.id)} - ${defaultDate}`;
    
    let queryString = 'requestType=FilterData';
    queryString += `&SGESTADO=${area.id}`;
    queryString += `&DTINI=${start}`;
    queryString += `&DTFIN=${end}`;
    
    Http.get(`/averageTimeAFGrid?${queryString}`).then(({ data }) => {
      gridGroup.datasource.updateData(data.dataset.averageTimeAFGridGroup);
    });
  }

  public closeAverageTimeAFModal() {
    this.AverageTimeAFModal.hide();
  }
  
  public getNomeEstado(sigla) {
    const estados = {
      AC: 'Acre',
      AL: 'Alagoas',
      AM: 'Amazonas',
      AP: 'Amapá',
      BA: 'Bahia',
      CE: 'Ceará',
      DF: 'Distrito Federal',
      ES: 'Espírito Santo',
      GO: 'Goiás',
      MA: 'Maranhão',
      MG: 'Minas Gerais',
      MS: 'Mato Grosso do Sul',
      MT: 'Mato Grosso',
      PA: 'Pará',
      PB: 'Paraíba',
      PE: 'Pernambuco',
      PR: 'Paraná',
      PI: 'Piauí',
      RJ: 'Rio de Janeiro',
      RN: 'Rio Grande do Norte',
      RO: 'Rondônia',
      RR: 'Roraima',
      RS: 'Rio Grande do Sul',
      SC: 'Santa Catarina',
      SE: 'Sergipe',
      SP: 'São Paulo',
      TO: 'Tocantins',
    };
    return estados[sigla];
  }
}
