import {
  Metadata,
  IEventParam,
  Http,
} from '@zeedhi/core';
import {
  Modal,
  ModalService,
  Select,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class ExceptionProfileModalController {
  private exceptionProfile!: Modal;

  public async createExceptionProfile() {
    this.exceptionProfile = await ModalService.createFromJson('/modals/exceptionProfileModal', true);
  }
  
  public async openExceptionProfile({ row, event }) {
    await this.exceptionProfile.show();
    event.stopPropagation();
    const exceptionGrid = Metadata.getInstance<TekGrid>('exceptionOperatorGrid').datasource;
    const sendObject = {
      NRPERFACLPAINEL: null,
      NRINDICPAINEL: null,
      NRPERFILACESSO: null,
      NRPAINEL: null,
    };
    if (row.DSINDICADOR) {
      (exceptionGrid as any).route = '/crudN/exceptionOperatorIndicator';
      sendObject.NRPERFACLPAINEL = row.NRPERFACLPAINEL;
      sendObject.NRINDICPAINEL = row.NRINDICPAINEL;
    } else {
      (exceptionGrid as any).route = '/crudN/exceptionOperatorPanel';
      sendObject.NRPERFACLPAINEL = row.NRPERFACLPAINEL;
      sendObject.NRPERFILACESSO = row.NRPERFILACESSO;
      sendObject.NRPAINEL = row.NRPAINEL;
    }
    exceptionGrid.setFilter(sendObject);
  }
  
  public closeExceptionProfile() {
    this.exceptionProfile.hide();
  }
  
  public deleteOperatorEx({ component, event }: IEventParam<TekGrid>) {
    event.preventDefault();
    const grid = Metadata.getInstance<TekGrid>('exceptionOperatorGrid');
    const nrperfilacesso = grid.datasource.filter.NRPERFILACESSO;
    const operator = component.selectedRows;
    if (nrperfilacesso) {
      const nrpainel = grid.datasource.filter.NRPAINEL;
      const cdfilial = Metadata.getInstance<Select>('CDFILIAL').value.CDFILIAL;
      const sendObject = {
        CDFILIAL: cdfilial,
        NRPERFILACESSO: nrperfilacesso,
        NRPAINEL: nrpainel,
        OPERADOR: operator,
      };
      Http.post('/deleteOperatorEx', {
        requestType: 'Row',
        row: sendObject,
      }).then(() => {
        grid.datasource.get();
      });
    } else {
      const sendObject = {
        NRINDICPAINEXCOPER: component.selectedRows,
      };
      Http.post('/deleteOperatorExInd', {
        requestType: 'Row',
        row: sendObject,
      }).then(() => {
        grid.datasource.get();
      });
    }
  }
}
